Program HostPar;

       {University of Malta Continuous Simulation System}
       {Author: J.Sklenar   January 1991     Version 2.1}

              {EXAMPLE: Hosts and Parasites}

  { The closed system containing two populations is simulated. }
  { The system is described by the following differential      }
  { equations:						       }
  {	       dHosts/dt =  K1*Hosts     - K3*Hosts*Parasites  }
  {	   dParasites/dt = -K2*Parasites + K4*Hosts*Parasites  }
  {							       }
  { where Hosts,Parasites are numbers of hosts and parasites,  }
  {	  K1 = 0.05/hour representes hosts reproduction,       }
  {	  K2 = 0.1/hour is parasites die off rate,	       }
  {	  K3 = 0.0002/parasite/hour representes hosts decrease }
  {		      caused by parasites,		       }
  {	  K4 = 0.0002/host/hour representes parasites increase }
  {		      caused by encounters with hosts.	       }
  {							       }
  { Initial values of numbers of hosts and parasites can be    }
  { inputed (start with 1000 and 500 respectively).	       }

Uses Graph, Crt;

{system constants}

const
   numpoint =	 700;	{maximum number of points to collect}
   numint   =	   1;	{number of integration methods}

{user defined constants}

   hscale   =      5;   {scaling hosts on screen}
   pscale   =      5;   {scaling parasites on screen}
   npoint   =    300;   {number of points to display}
   dx       =      2;   {step of the x axis to plot}


{system types}

type
   collectr = array[1..numpoint] of real;    {to collect real values}
   collecti = array[1..numpoint] of integer; {to collect int. values}
   numpoit  = 1..numpoint;                   {for indexes of arrays}

{user defined types}

{system variables}

var
   time     : real;      {model time}
   exper    : byte;      {experiment number}
   countpr  : real;      {counter for printing}
   countpl  : real;      {counter for plotting}
   fintim   : real; 	 {experiment length}
   prdel    : real;	 {printing interval}
   outdel   : real;	 {plot interval}
   delt     : real;	 {integration step}
   numrep   : byte;	 {number of repetitions}


{user defined variables}

   host,paras     : real;     {number of hosts and parasites}
   ihost,iparas   : integer;  {initial values}
   hostpl,paraspl : collectr; {arrays for collection}
   colpl          : numpoit;  {index}
   prod           : real;     {product hosts*parasites}

{system procedures and functions}

   function step (t:real):integer;       {step of CSMP type}
   begin
     if (time >= t) then step:=1 else step:=0
   end;

   procedure integral(input:real;var output:real);
     {integration methods:    }
     {  so far only Euler     }
   begin
     output := output + delt*input
   end;

{user defined procedures and functions}
{must include:                        }
{ initiation = first model initiation }
{ modelinit  = experiment initiation  }
{ simulation = one simulation step    }
{ printing   = print/collect values   }
{ plotting   = plot/collect values    }
{ expter     = termination test       }
{ modelup    = model updating         }
{ expeval    = experiment evaluation  }
{ termination= final actions          }


  procedure initiation; { default values }
  begin     
   fintim  :=  300.0;	{experiment length}
   prdel   :=     20;	{printing interval}
   outdel  :=	 1.0;	{plot interval}
   delt    :=	 0.1;	{integration step}
   numrep  :=	   2;	{number of repetitions}
   ihost   :=   1000;   {initial number of hosts}
   iparas  :=    500;   {initial number of parasites}   
  end;


  procedure modelinit;
  var i:integer;
      c:char;
  begin
   Textbackground(blue);
   Textcolor(yellow);
   RestoreCrtMode;
   ClrScr;
   GotoXY(15,2);
   writeln('MODEL INITIALIZATION');
   GotoXY(1,5);
   write('(1) Enter experiment length:  ');
    write(fintim:13:2);
   GotoXY(1,7);
   write('(2) Enter printing interval:  ');
    write(prdel:13:2);
   GotoXY(1,9);
   write('(3) Enter plot interval:      ');
    write(outdel:13:2);
   GotoXY(1,11);
   write('(4) Enter integration step:   ');
    write(delt:13:2);
   GotoXY(1,13);
   write('(5) Enter number of repetitions: ');
    write(numrep:10);
   GotoXY(1,15);
   write('(6) Enter initial number of hosts: ');
    write(ihost:8);
   GoToXY(1,17);
   write('(7) Enter initial number of parasites:');
    write(iparas:5);
   GoToXY(1,19);
   write('(8) Start Simulation');

  repeat      { reading parameters }

   repeat
      GotoXY(1,23); write('Enter option : ');
      ClrEol; c := Readkey;
   until (c>'0') and (c<'9');

   case c of
   '1': begin GotoXY(45,5); ClrEol; readln(fintim);
              GotoXY(45,5); write(fintim:9:2); ClrEol;
        end;
   '2': begin GotoXY(45,7); ClrEol; readln(prdel);
              GotoXY(45,7); write(prdel:9:2); ClrEol;
        end;
   '3': begin GotoXY(45,9); ClrEol; readln(outdel);
              GotoXY(45,9); write(outdel:9:2); ClrEol;
        end;
   '4': begin GotoXY(45,11); ClrEol; readln(delt);
              GotoXY(45,11); write(delt:9:2); ClrEol;
        end;
   '5': begin GotoXY(45,13); ClrEol; readln(numrep);
              GotoXY(45,13); write(numrep:9); ClrEol;
        end;
   '6': begin GotoXY(45,15); ClrEol; readln(ihost);
              GotoXY(45,15); write(ihost:9); ClrEol;
        end;
   '7': begin GotoXY(45,17); ClrEol; readln(iparas);
              GotoXY(45,17); write(iparas:9); ClrEol;
        end;

   '8':;
   end; {case}

  until c='8';
      host:=ihost;
      paras:=iparas;
      Textbackground(cyan);
      Textcolor(yellow);
      ClrScr;
      writeln('      Time         Hosts     Parasites');
      writeln('  ************************************');
      time:=0;
      colpl:=1;
      for i:=1 to numpoint do
      begin
         hostpl[i]:=0;
         paraspl[i]:=0
      end
   end;

   procedure modelupdate;
   begin end;

   procedure termination;
   begin
      RestoreCrtMode;
   end;

   procedure simulation;
   begin

   {multiplication} 

      prod:=host*paras;

   {integration for hosts}

      integral(-0.0002*prod + 0.05*host,host);

   {integration for parasites}

      integral( 0.0002*prod - 0.1*paras,paras);
   end;

   procedure printing;
   begin
      writeln(time:10:2,host:14:1,paras:14:1)
   end;

   procedure plotting;
   begin
      hostpl[colpl]:=host;
      paraspl[colpl]:=paras;
      colpl:=colpl+1
   end;

   function expter: boolean;
   begin
      expter:=false
   end;

   procedure expeval;
   var i,x,y,h: integer;
       Graphdriver,Graphmode:integer;
   begin
      writeln('press Enter to plot');
      readln;
      Graphdriver := Detect;
      InitGraph(Graphdriver,Graphmode,'c:\tp\bgi');
      ClearViewPort;
      SetBkColor(black);
      SetColor(yellow);
      h := GetMaxY div 5;
      SetTextStyle(DefaultFont, HorizDir, 2);
      OutTextXY(5, h div 2, 'Hosts & Parasites');
      x := 0;                         { x,y = origin }
      y := h*4;
      Line(x,y,GetMaxX,y);
      Moveto(x,y-round(ihost/hscale));
      SetColor(red);
      OutText('H');
      x := GetX;
      for i:=1 to npoint do
      begin
         x:=x+dx; Lineto(x,y-round(hostpl[i]/hscale));
      end;
      x := 0;                         { x,y = origin }
      y := h*4;
      Moveto(x,y-round(iparas/pscale));
      SetColor(blue);
      OutText('P');
      x := GetX;
      for i:=1 to npoint do
      begin
         x:=x+dx; Lineto(x,y-round(paraspl[i]/pscale));
      end;
      Moveto(0,y+h div 2);
      SetColor(yellow);
      SetTextStyle(DefaultFont, HorizDir, 1);
      OutText('Press Enter to continue');
      readln
   end;


begin  {of the program body}
   initiation;
   repeat
      modelinit;
      countpr:=0;
      countpl:=0;
      repeat
         time:=time+delt;
         countpr:=countpr+delt;
         countpl:=countpl+delt;
         simulation;
         if countpr>=prdel then
         begin
            printing;
            countpr:=0
         end;
         if countpl>=outdel then
         begin
            plotting;
            countpl:=0
         end;
     until (time>=fintim) or (expter);
     expeval;
     modelupdate;
     Dec(numrep);
   until numrep = 0;
   termination;
end.


