Program Rocket1;

    {University of Malta Continuous Simulation System}
    {Author: J.Sklenar	 January 1992     Version 2.1}

       { Experiment 1 : rocket, fixed number of }
       { experiments, possibility to change the }
       { initial values.			}


{ This is the simple system for computer simulation of	  }
{ dynamic systems. The system simulated may be expressed  }
{ as a block diagram or a set of equations. The user pro- }
{ grams the activity in one simulation step. The algorithm}
{ is like that: 					  }
{	   system_initiation				  }
{	   for all experiments do			  }
{		      model_initiation			  }
{		      repeat				  }
{			  time := time + delta_t	  }
{			  simulation_step_activity	  }
{			  print if necessary		  }
{			  plot	if necessary		  }
{		      until termination_condition	  }
{		      experiment_evaluation		  }
{		      model_updating			  }
{	   termination					  }
{							  }
{notes: - time is expressed as a real variable in abstract}
{	  units (system variable time), 		  }
{	- user is given some standard constants,types,	  }
{	  variables,procedures and functions. Definition  }
{	  of application dependent objects is supposed,   }
{	- we suggest to store the plotted data in arrayes }
{	  (of type collectr or collecti) and to draw	  }
{	  graphs after the completion of the experiment,  }
{							  }
{Example: The system simulated is the model rocket. The   }
{   engine fires in three stages, each stage takes 1 sec. }
{   The constant accelerations of the stages are IACC,40  }
{   and 30 m/s*s. The stages fire at times 0,10 and 15 s. }
{   Between stages there is the gravity -9.8 m/s*s.	  }
{   Equations:						  }
{    altitude = integral(velocity) + initial_altitude	  }
{    velocity = integral(acceleration) + initial_velocity }
{   The simulation finishes after 100 sec or if altitude  }
{   equals zero.					  }

Uses Graph, Crt;

{system constants: }

const
   fintim   = 200.0;   {experiment length}
   prdel    =	2.0;   {printing interval}
   outdel   =	1.0;   {plot interval}
   delt     =  0.05;   {integration step}
   numpoint =	300;   {maximum number of points to collect}
   numrep   =	  2;   {number of repetitions}
   numint   =	  1;   {number of integration methods}

{user defined constants: }

   ascale   =     2;   {Scaling altitude on screen}
   xstep    =    10;   {Step of x axis for plot}
   numx     =    69;   {Number of points}

{system types: }

type
   collectr = array[1..numpoint] of real; {to collect real values}
   collecti = array[1..numpoint] of integer; { -"- integer values}
   numrept  = 1..numrep;     {to number experiments}
   numpoit  = 1..numpoint;   {for indexes of arrays}

{user defined types: }

{system variables: }

var
   time     : real;      {model time}
   exper    : numrept;   {experiment number}
   countpr  : real;      {counter for printing}
   countpl  : real;      {counter for plotting}

{user defined variables: }

   alt,vel,acc : real;     {altitude,velocity,acceleration}
   ialt,ivel,iacc : real;  {initial values}
   altpl,velpl : collectr; {arrays for collection}
   colpl : numpoit;        {index}
   h     : word;

{system procedures and functions: }

   function step (t:real):integer;  {step of CSMP type}
   begin
     if (time >= t) then step:=1 else step:=0
   end;

   procedure integral(input:real;var output:real);
     {integration methods:    }
     {  so far only Euler     }
   begin
     output := output + delt*input
   end;

{user defined procedures and functions}
{must include:                        }
{ initiation  = first model initiation }
{ modelinit   = experiment initiation  }
{ simulation  = one simulation step    }
{ printing    = print/collect values   }
{ plotting    = plot/collect values    }
{ expter      = termination test       }
{ modelupdate = model updating         }
{ expeval     = experiment evaluation  }
{ termination = final actions          }


   procedure initiation;
   begin end;

  procedure modelinit;
  var i:integer;
  begin
   Textbackground(blue);
   Textcolor(yellow);
   RestoreCrtMode;
   ClrScr;
   GotoXY(15,2);
   writeln('MODEL INITIALIZATION');
   GotoXY(1,5);
   write('Enter the initial altitude:      ');
   readln(ialt);
   GotoXY(1,7);
   write('Enter the initial velocity:      ');
   readln(ivel);
   GotoXY(1,9);
   write('Enter the initial acceleration:  ');
   readln(iacc);

      Textbackground(cyan);
      Textcolor(yellow);
      ClrScr;
      writeln('      Time      Altitude      Velocity  Acceleration');
      writeln('  **************************************************');

      alt:=ialt;
      vel:=ivel;
      time:=0;
      colpl:=1;
      for i:=1 to numpoint do
      begin
         altpl[i]:=0;
         velpl[i]:=0
      end;
   end;

   procedure modelupdate;
   begin end;

   procedure termination;
   begin
      RestoreCrtMode;
   end;

   procedure simulation;
   begin

    { at first compute the acceleration using the step function: }

      acc:=iacc-(iacc+9.8)*step(1.0)+49.8*step(10.0)
           -49.8*step(11.0)+39.8*step(15.0)-39.8*step(16.0);

    { then compute the velocity as the output of the integrator  }
    { the input of which is the acceleration:                    }

      integral(acc,vel);

    { altitude is the integral of velocity: }

      integral(vel,alt);

    { altitude must be nonnegative: }

      if (alt<0) then alt:=0
   end;

   procedure printing;
   begin
      writeln(time:10:2,alt:14:1,vel:14:1,acc:14:1)
   end;

   procedure plotting;
   begin
      altpl[colpl]:=alt;
      colpl:=colpl+1
   end;

   function expter: boolean;
   begin
      expter:=(alt=0)
   end;

   procedure expeval;
   var i,x,y:integer;
       graphdriver,graphmode:integer;
   begin
      writeln('press Enter to plot');
      readln;
      graphdriver:=Detect;
      InitGraph(graphdriver,graphmode,'c:\tp\bgi');
      SetBkColor(black);
      SetColor(yellow);
      h := GetMaxY div 5;
      SetTextStyle(DefaultFont, HorizDir, 2);
      OutTextXY(5, h div 2, 'Rocket Flight');
      x:=0;              { origin }
      y:=h*4;
      SetColor(red);
      Moveto(x,y-round(ialt/ascale));
      SetTextStyle(DefaultFont, HorizDir, 1);
      OutText('Initial Altitude');
      Moveto(x,y-round(ialt/ascale));
      for i:=1 to numx do
      begin
         x:=x+xstep;
         Lineto(x,y-round(altpl[i]/ascale));
      end;
      Moveto(0, y+h div 2);
      SetColor(yellow);
      OutText('Press Enter to continue');
      readln
   end;


begin                {of the program body}
   initiation;
   for exper:=1 to numrep do
   begin
      modelinit;
      countpr:=0;
      countpl:=0;
      repeat
         time:=time+delt;
         countpr:=countpr+delt;
         countpl:=countpl+delt;
         simulation;
         if countpr>=prdel then
         begin
            printing;
            countpr:=0
         end;
         if countpl>=outdel then
         begin
            plotting;
            countpl:=0
         end;
     until (time>=fintim) or (expter);
     expeval;
     modelupdate;
   end;            {of experiment}
   termination
end.


