Program Rocket5;

    {University of Malta Continuous Simulation System}

    {Author: J.Sklenar	                February 1992}

       { Experiment 5 : rocket with manual control, }
       { 2 dimensions, no gravity.                  }
       {          !!!  ANIMATION  !!!               }

{ This is the simple system for computer simulation of	  }
{ dynamic systems. The system simulated may be expressed  }
{ as a block diagram or a set of equations. The user pro- }
{ grams the activity in one simulation step. The algorithm}
{ is like that: 					  }
{	   system_initiation				  }
{	   for all experiments do			  }
{		      model_initiation			  }
{		      repeat				  }
{			  time := time + delta_t	  }
{			  simulation_step_activity	  }
{			  print if necessary		  }
{			  plot	if necessary		  }
{		      until termination_condition	  }
{		      experiment_evaluation		  }
{		      model_updating			  }
{	   termination					  }
{							  }
{notes: - time is expressed as a real variable in abstract}
{	  units (system variable time), 		  }
{	- user is given some standard constants,types,	  }
{	  variables,procedures and functions. Definition  }
{	  of application dependent objects is supposed,   }
{	- we suggest to store the plotted data in arrayes }
{	  (of type collectr or collecti) and to draw	  }
{	  graphs after the completion of the experiment.  }
{							  }
{ EXAMPLE: The system simulated is a space ship moving in }
{   2 dimensional space. There are 4 jets giving accelera-}
{   tion in 4 directions: +/-x, +/-y. The jets are started}
{   and stopped by the following keys:                    }

{             s  =  +x (East)                             }
{             a  =  -x (West)                             }
{             k  =  +y (North)                            }
{             m  =  -y (South)                            }

{   Engines fire until the whole fuel is consumed. The    }
{   amount of fuel (expressed in units of firing per jet) }
{   and the constant acceleration of jets are entered.    }

{   Equations:	( for both coordinates )                  }
{    position = integral(velocity) + initial_position     }
{    velocity = integral(acceleration) + initial_velocity }

{    Press "Q" to stop the simulation.                    }

{$N+,E+}
Uses Graph, Crt;

{system constants: }

const
   delt     =  0.05;   {integration step}
   numrep   =	  3;   {number of repetitions}
   numint   =	  1;   {number of integration methods}

{user defined constants: }

   scX      =     6;   {Scaling X position on screen}
   scY      =     4;   {Scaling Y position on screen}
   r        =    24;   {Rocket size}

{system types: }

type
   numrept  = 1..numrep;     {to number experiments}

{user defined types: }

{system variables: }

var
   time     : real;      {model time}
   exper    : numrept;   {experiment number}

{user defined variables: }

   xpos,xvel,xacc,ypos,yvel,yacc : real; {Position,velocity,acceleration}
   acE,acW,acN,acS : real; {Accelerations of jets}
                           {Initial position is X0,Y0 }
                           {Initial velocities and accelerations are zero}
   iacc : real;            {Acceleration of jets - all are the same}
   graphdriver,graphmode:integer;   {used by graphics}
   X0,Y0 : word;           {Coordinates for Get/PutImage (previous position)}
   Size : longint;         {Memory size  for Get/PutImage}
   Rock,FireE,FireW,FireN,FireS : pointer;    {Pointers to images}
   firE,firW,firN,firS : boolean;   {Fires displayed: yes/no}
   ifuel : real;           {Initial amount of fuel - 1 unit/sec}
   fuel : real;            {Amount of fuel}
   c : char;               {Character read}

{system procedures and functions: }

   procedure integral(input:real;var output:real);
     {integration methods:    }
     {  so far only Euler     }
   begin
     output := output + delt*input
   end;

{user defined procedures and functions }
{must include:                         }
{ initiation  = first model initiation }
{ modelinit   = experiment initiation  }
{ simulation  = one simulation step    }
{ printing    = print/collect values   }
{ plotting    = plot/collect values    }
{ expter      = termination test       }
{ modelupdate = model updating         }
{ expeval     = experiment evaluation  }
{ termination = final actions          }

 procedure Drawrocket(xr,yr,r: word);
 begin
    SetColor(lightblue);
    SetFillStyle(SolidFill, Blue);
    Bar(xr,yr,xr+r,yr+r);
 end;

 procedure DrawfireE(xf,yf,r: word);
 begin
    SetColor(lightred);
    MoveTo(xf, yf + r div 8 * 3);
    LineRel(0, r div 4);
    LineRel(r, r div 8);
    LineRel(0, - r div 2);
    LineTo(xf, yf + r div 8 * 3);
    SetFillStyle(SolidFill, red);
    FloodFill(xf + r div 2, yf + r div 2, lightred);
 end;

  procedure DrawfireW(xf,yf,r: word);
 begin
    SetColor(lightred);
    MoveTo(xf+r, yf + r div 8 * 3);
    LineRel(0, r div 4);
    LineRel(-r, r div 8);
    LineRel(0, - r div 2);
    LineTo(xf+r, yf + r div 8 * 3);
    SetFillStyle(SolidFill, red);
    FloodFill(xf + r div 2, yf + r div 2, lightred);
 end;

 procedure DrawfireN(xf,yf,r: word);
 begin
    SetColor(lightred);
    MoveTo(xf + r div 8 * 3,yf+r);
    LineRel(r div 4,0);
    LineRel(r div 8, - r);
    LineRel(- r div 2,0);
    LineTo(xf + r div 8 * 3,yf+r);
    SetFillStyle(SolidFill, red);
    FloodFill(xf + r div 2, yf + r div 2, lightred);
 end;

 procedure DrawfireS(xf,yf,r: word);
 begin
    SetColor(lightred);
    MoveTo(xf + r div 8 * 3,yf);
    LineRel(r div 4,0);
    LineRel(r div 8,r);
    LineRel(- r div 2,0);
    LineTo(xf + r div 8 * 3,yf);
    SetFillStyle(SolidFill, red);
    FloodFill(xf + r div 2, yf + r div 2, lightred);
 end;

 procedure Status;   {Rocket status}
 begin
   GotoXY(1,1);
   write('Position: X =',(xpos - (GetMaxX div 2)*scX):7:1);
   write('  Y =',((GetMaxY div 2)*scY - ypos):7:1);
   GotoXY(1,3);
   write('Velocity: X =',xvel:7:1,'  Y =',-yvel:7:1);
   GotoXY(1,5);
   write('Acceler.: X =',xacc:7:1,'  Y =',-yacc:7:1);
   GoToXY(1,8);
   write('Remaining fuel: ',fuel:4:1);
 end;

  procedure initiation;
  begin
   DirectVideo := false;
   graphdriver:=Detect;
   InitGraph(graphdriver,graphmode,'c:\tp\bgi');
   SetBkColor(black);
   X0 := GetMaxX div 2;
   Y0 := GetMaxY div 2;

      {Draw and save rocket image:}
   Drawrocket(X0,Y0,r);
   Size := Imagesize(X0,Y0,X0+r,Y0+r);
   GetMem(Rock,Size);
   GetImage(X0,Y0,X0+r,Y0+r,Rock^);
   PutImage(X0,Y0,Rock^,XORput);

      {Draw and save images of fires:}
   DrawfireE(X0+r,Y0,r);
   GetMem(FireE,Size);
   GetImage(X0+r,Y0,X0+r+r,Y0+r,FireE^);
   PutImage(X0+r,Y0,FireE^,XORput);

   DrawfireW(X0-r,Y0,r);
   GetMem(FireW,Size);
   GetImage(X0-r,Y0,X0,Y0+r,FireW^);
   PutImage(X0-r,Y0,FireW^,XORput);

   DrawfireN(X0,Y0-r,r);
   GetMem(FireN,Size);
   GetImage(X0,Y0-r,X0+r,Y0,FireN^);
   PutImage(X0,Y0-r,FireN^,XORput);

   DrawfireS(X0,Y0+r,r);
   GetMem(FireS,Size);
   GetImage(X0,Y0+r,X0+r,Y0+r+r,FireS^);
   PutImage(X0,Y0+r,FireS^,XORput);

   Textbackground(blue);
   Textcolor(yellow);
   RestoreCrtMode;
   ClrScr;
   writeln;
   writeln('   The system simulated is a space ship moving in two');
   writeln('   dimensional  space.  There  are  four  jets giving');
   writeln('   acceleration  in four directions:  +x, -x, +y, -y. ');
   writeln('   The jets are started by the following keys:');
   writeln;
   writeln('             s  =  +x (East)');
   writeln('             a  =  -x (West)');
   writeln('             k  =  +y (North)');
   writeln('             m  =  -y (South)');
   writeln;
   writeln('   Coordinates have the origin in the middle of the screen.');
   writeln('   A key either stops the jet for the opposite direction or');
   writeln('   it starts the proper jet immediatelly. If the opposite');
   writeln('   jet fires, it is necessary to press a key twice.');
   writeln('   Engines fire until the whole fuel is consumed.');
   writeln;
   writeln('   Press "Q" to stop the simulation (you have three runs).');
   writeln;
   writeln('   Press Enter to continue');
   readln;
   ClrScr;
   GotoXY(15,2);
   writeln('MODEL INITIALIZATION');
   GotoXY(1,5);
   write(' Enter the acceleration of jets (try 40) : ');
   readln(iacc);
   GotoXY(1,8);
   write(' Enter the amount of fuel (try 200)      : ');
   readln(ifuel);
  end;

  procedure modelinit;
  var i:integer;
  begin
    SetGraphMode(graphmode);
    SetBkColor(black);
    X0 := GetMaxX div 2;              {initial position}
    Y0 := GetMaxY div 2;
    PutImage(X0,Y0,Rock^,XORput);     {rocket, no firing}
      firE:=false;
      firW:=false;
      firN:=false;
      firS:=false;
      fuel:=ifuel;
      acE:=0;
      acW:=0;
      acN:=0;
      acS:=0;
      xpos:=X0*scX;
      ypos:=Y0*scY;
      xvel:=0;
      yvel:=0;
      xacc:=0;
      yacc:=0;
      time:=0;
      c:=' ';
   end;

   procedure modelupdate;
   begin end;

   procedure termination;
   begin
      RestoreCrtMode;
   end;

   procedure simulation;
   begin

    { compute velocities as outputs of integrators: }

      integral(xacc,xvel);
      integral(yacc,yvel);

    { positions are integrals of velocities: }

      integral(xvel,xpos);
      integral(yvel,ypos);

    { show the rocket at the new position: }

      PutImage(X0,Y0,Rock^,XORput); {clear the previous rocket}
      if firE then begin
        PutImage(X0+r,Y0,FireE^,XORput); {clear the previous fire E}
        firE:=false;
      end;
      if firW then begin
        PutImage(X0-r,Y0,FireW^,XORput); {clear the previous fire W}
        firW:=false;
      end;
      if firN then begin
        PutImage(X0,Y0-r,FireN^,XORput); {clear the previous fire N}
        firN:=false;
      end;
      if firS then begin
        PutImage(X0,Y0+r,FireS^,XORput); {clear the previous fire S}
        firS:=false;
      end;

      X0:=round(xpos/scX);        {new position}
      Y0:=round(ypos/scY);

      PutImage(X0,Y0,Rock^,XORput); {new rocket}
      if acE>0 then begin
        fuel:=fuel-delt;
        PutImage(X0+r,Y0,FireE^,XORput); {new fire E}
        firE:=true;
      end;
      if acW>0 then begin
        fuel:=fuel-delt;
        PutImage(X0-r,Y0,FireW^,XORput); {new fire W}
        firW:=true;
      end;
      if acN>0 then begin
        fuel:=fuel-delt;
        PutImage(X0,Y0-r,FireN^,XORput); {new fire N}
        firN:=true;
      end;
      if acS>0 then begin
        fuel:=fuel-delt;
        PutImage(X0,Y0+r,FireS^,XORput); {new fire S}
        firS:=true;
      end;
      if fuel<=0 then begin
          fuel:=0;
          acE:=0;
          acW:=0;
          acN:=0;
          acS:=0;
      end;

    { Compute the new accelerations : }

    if Keypressed then begin
     c:=Readkey;
     if fuel>0 then begin
         if (c='S')or(c='s') then
            if acE>0 then acE:=0 else acW:=iacc;
         if (c='A')or(c='a') then
            if acW>0 then acW:=0 else acE:=iacc;
         if (c='K')or(c='k') then
            if acN>0 then acN:=0 else acS:=iacc;
         if (c='M')or(c='m') then
            if acS>0 then acS:=0 else acN:=iacc;
     end{if}
    end{if};

      xacc:=acW-acE;   {Resulting accelerations in both directions}
      yacc:=acN-acS;   {  (for the next step)  }
      Status;          {Displaying rocket status}
      Delay(3);
   end;

   function expter: boolean;
   begin
      expter:=(c='Q')or(c='q')
   end;

{   procedure expeval;
   var i,x,y,h:integer;
   begin
      GoToXY(2,16);
      write('Maximum altitude = ', maxalt:10:2,' metres');
      GoToXY(2,18);
      write('Flight duration  = ', time:10:2,' seconds');
      GoToXY(2,23);
      write('Press Enter to plot');
      readln;
      SetGraphMode(graphmode);
      SetBkColor(black);
      SetColor(yellow);
      h := GetMaxY div 5;
      SetTextStyle(DefaultFont, HorizDir, 2);
      OutTextXY(370, h div 2, 'Rocket Flight');
      GotoXY(40,8);
      write('Maximum altitude = ', maxalt:9:1,' metres');
      GotoXY(40,10);
      write('Flight duration  = ', time:9:1,' seconds');
      x:=0;   
      y:=GetMaxY-10;
      SetColor(red);
      Moveto(x,y-round(ialt/ascale));
      SetTextStyle(DefaultFont, HorizDir, 1);
      OutText('Initial Altitude');
      Moveto(x,y-round(ialt/ascale));
      for i:=1 to numx do
      begin
         x:=x+xstep;
         LineTo(x,y-round(altpl[i]/ascale));
      end;
      Moveto(400, y - h div 2);
      SetColor(yellow);
      OutText('Press Enter to continue');
      readln
  end;
}

begin                {of the program body}
   initiation;
   for exper:=1 to numrep do
   begin
     modelinit;
     repeat
         time:=time+delt;
         simulation;
     until (expter);
     {expeval;}
     modelupdate;
   end;            {of experiment}
   termination
end.

